@@ -151,7 +151,7 @@ send_selection_notify (GsdClipboardManager *manager,
                     (XEvent *)&notify);
         XSync (manager->priv->display, False);
 
-        gdk_error_trap_pop ();
+        gdk_error_trap_pop_ignored ();
 }
 
 static void
@@ -178,7 +178,7 @@ finish_selection_request (GsdClipboardManager *manager,
                     False, NoEventMask, (XEvent *) &notify);
         XSync (manager->priv->display, False);
 
-        gdk_error_trap_pop ();
+        gdk_error_trap_pop_ignored ();
 }
 
 static int
@@ -554,7 +554,7 @@ convert_clipboard_target (IncrConversion      *rdata,
 
                         XSync (manager->priv->display, False);
 
-                        gdk_error_trap_pop ();
+                        gdk_error_trap_pop_ignored ();
                 }
         }
 }
@@ -825,11 +825,11 @@ clipboard_manager_watch_cb (GsdClipboardManager *manager,
         GdkDisplay *display;
 
         display = gdk_display_get_default ();
-        gdkwin = gdk_window_lookup_for_display (display, window);
+        gdkwin = gdk_x11_window_lookup_for_display (display, window);
 
         if (is_start) {
                 if (gdkwin == NULL) {
-                        gdkwin = gdk_window_foreign_new_for_display (display, window);
+                        gdkwin = gdk_x11_window_foreign_new_for_display (display, window);
                 } else {
                         g_object_ref (gdkwin);
                 }
