@@ -25,8 +25,10 @@
 
 #define SETTINGS_BINDING_DIR "org.gnome.settings-daemon.plugins.media-keys"
 
-enum {
+typedef enum {
         TOUCHPAD_KEY,
+        TOUCHPAD_ON_KEY,
+        TOUCHPAD_OFF_KEY,
         MUTE_KEY,
         VOLUME_DOWN_KEY,
         VOLUME_UP_KEY,
@@ -55,8 +57,11 @@ enum {
         MAGNIFIER_KEY,
         SCREENREADER_KEY,
         ON_SCREEN_KEYBOARD_KEY,
+        INCREASE_TEXT_KEY,
+        DECREASE_TEXT_KEY,
+        TOGGLE_CONTRAST_KEY,
         HANDLED_KEYS
-};
+} MediaKeyType;
 
 static struct {
         int key_type;
@@ -65,6 +70,8 @@ static struct {
         Key *key;
 } keys[HANDLED_KEYS] = {
         { TOUCHPAD_KEY, "touchpad", NULL, NULL },
+	{ TOUCHPAD_ON_KEY, NULL, "XF86TouchpadOn", NULL },
+	{ TOUCHPAD_OFF_KEY, NULL, "XF86TouchpadOff", NULL },
         { MUTE_KEY, "volume-mute",NULL, NULL },
         { VOLUME_DOWN_KEY, "volume-down", NULL, NULL },
         { VOLUME_UP_KEY, "volume-up", NULL, NULL },
@@ -96,6 +103,9 @@ static struct {
 	{ MAGNIFIER_KEY, "magnifier", NULL, NULL },
 	{ SCREENREADER_KEY, "screenreader", NULL, NULL },
 	{ ON_SCREEN_KEYBOARD_KEY, "on-screen-keyboard", NULL, NULL },
+	{ INCREASE_TEXT_KEY, "increase-text-size", NULL, NULL },
+	{ DECREASE_TEXT_KEY, "decrease-text-size", NULL, NULL },
+	{ TOGGLE_CONTRAST_KEY, "toggle-contrast", NULL, NULL },
 };
 
 #endif /* __ACME_H__ */
