@@ -41,7 +41,6 @@ struct GvcMixerSinkInputPrivate
 static void     gvc_mixer_sink_input_class_init (GvcMixerSinkInputClass *klass);
 static void     gvc_mixer_sink_input_init       (GvcMixerSinkInput      *mixer_sink_input);
 static void     gvc_mixer_sink_input_finalize   (GObject                *object);
-static void     gvc_mixer_sink_input_dispose    (GObject                *object);
 
 G_DEFINE_TYPE (GvcMixerSinkInput, gvc_mixer_sink_input, GVC_TYPE_MIXER_STREAM)
 
@@ -53,12 +52,10 @@ gvc_mixer_sink_input_push_volume (GvcMixerStream *stream, gpointer *op)
         const GvcChannelMap *map;
         pa_context          *context;
         const pa_cvolume    *cv;
-        guint                num_channels;
 
         index = gvc_mixer_stream_get_index (stream);
 
         map = gvc_mixer_stream_get_channel_map (stream);
-        num_channels = gvc_channel_map_get_num_channels (map);
 
         cv = gvc_channel_map_get_cvolume(map);
 
@@ -107,29 +104,12 @@ gvc_mixer_sink_input_change_is_muted (GvcMixerStream *stream,
         return TRUE;
 }
 
-static GObject *
-gvc_mixer_sink_input_constructor (GType                  type,
-                                  guint                  n_construct_properties,
-                                  GObjectConstructParam *construct_params)
-{
-        GObject       *object;
-        GvcMixerSinkInput *self;
-
-        object = G_OBJECT_CLASS (gvc_mixer_sink_input_parent_class)->constructor (type, n_construct_properties, construct_params);
-
-        self = GVC_MIXER_SINK_INPUT (object);
-
-        return object;
-}
-
 static void
 gvc_mixer_sink_input_class_init (GvcMixerSinkInputClass *klass)
 {
         GObjectClass        *object_class = G_OBJECT_CLASS (klass);
         GvcMixerStreamClass *stream_class = GVC_MIXER_STREAM_CLASS (klass);
 
-        object_class->constructor = gvc_mixer_sink_input_constructor;
-        object_class->dispose = gvc_mixer_sink_input_dispose;
         object_class->finalize = gvc_mixer_sink_input_finalize;
 
         stream_class->push_volume = gvc_mixer_sink_input_push_volume;
@@ -144,19 +124,6 @@ gvc_mixer_sink_input_init (GvcMixerSinkInput *sink_input)
         sink_input->priv = GVC_MIXER_SINK_INPUT_GET_PRIVATE (sink_input);
 }
 
-static void
-gvc_mixer_sink_input_dispose (GObject *object)
-{
-        GvcMixerSinkInput *mixer_sink_input;
-
-        g_return_if_fail (object != NULL);
-        g_return_if_fail (GVC_IS_MIXER_SINK_INPUT (object));
-
-        mixer_sink_input = GVC_MIXER_SINK_INPUT (object);
-
-        G_OBJECT_CLASS (gvc_mixer_sink_input_parent_class)->dispose (object);
-}
-
 static void
 gvc_mixer_sink_input_finalize (GObject *object)
 {
