@@ -288,7 +288,7 @@ touchpad_has_single_button (XDevice *device)
         if (rc == Success)
                 XFree (data);
 
-        gdk_error_trap_pop ();
+        gdk_error_trap_pop_ignored ();
 
         return is_single_button;
 }
@@ -539,7 +539,7 @@ set_middle_button (GsdMouseManager *manager,
                         XChangeDeviceProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
                                                device, prop, type, format, PropModeReplace, data, nitems);
 
-                        gdk_error_trap_pop ();
+                        gdk_error_trap_pop_ignored ();
                 }
 
                 XFree (data);
@@ -555,7 +555,7 @@ set_disable_w_typing (GsdMouseManager *manager, gboolean state)
 {
         if (state && touchpad_is_present ()) {
                 GError *error = NULL;
-                char *args[5];
+                char *args[6];
 
                 if (manager->priv->syndaemon_spawned)
                         return 0;
@@ -563,8 +563,9 @@ set_disable_w_typing (GsdMouseManager *manager, gboolean state)
                 args[0] = "syndaemon";
                 args[1] = "-i";
                 args[2] = "2.0";
-                args[3] = "-k";
-                args[4] = NULL;
+                args[3] = "-K";
+                args[4] = "-R";
+                args[5] = NULL;
 
                 if (!g_find_program_in_path (args[0]))
                         return 0;
