@@ -649,7 +649,7 @@ gsd_xrandr_manager_2_apply_configuration (GsdXrandrManager *manager,
         gboolean result;
 
         if (parent_window_id != 0)
-                parent_window = gdk_x11_window_foreign_new_for_display (gdk_display_get_default (), (GdkNativeWindow) parent_window_id);
+                parent_window = gdk_x11_window_foreign_new_for_display (gdk_display_get_default (), (Window) parent_window_id);
         else
                 parent_window = NULL;
 
